"""
    This file is part of Accountant.

    Accountant is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with Accountant.  If not, see <http://www.gnu.org/licenses/>.
"""
from . import db

from .accounts import Account
from .scheduled_operations import ScheduledOperation

from sqlalchemy import func, desc
from sqlalchemy.orm import column_property
from sqlalchemy.sql import func, select

class Operation(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    pointed = db.Column(db.Boolean, nullable = False, default = False)
    operation_date = db.Column(db.Date, nullable = False)
    label = db.Column(db.String(500), nullable = False)
    value = db.Column(db.Numeric(15, 2), nullable = False)
    account_id = db.Column(db.Integer, db.ForeignKey('account.id'))
    scheduled_operation_id = db.Column(db.Integer, db.ForeignKey('scheduled_operation.id'))

    account = db.relationship(Account, backref = db.backref('operation', lazy="dynamic"))

    category = db.Column(db.String(100), nullable = True)

    canceled = db.Column(db.Boolean, nullable = False)

    def __init__(self, pointed, label, value, account_id, operation_date = None, category = None):
        self.pointed = pointed
        self.operation_date = operation_date
        self.label = label
        self.value = value
        self.account_id = account_id
        self.category = category

